/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.util.ArrayList;
import java.util.List;

import org.easymock.EasyMock;
import org.junit.ClassRule;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.SubDomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.NoContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.SubdomainUnknownException;
import org.ow2.petals.registry_overlay.client.configuration.ConfigurationBuilder;
import org.ow2.petals.registry_overlay.junit.PetalsRegistryOverlayNode;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * Abstract class for unit tests of {@link RegistryOverlayImpl}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AbstractRegistryOverlayImplTest {

    /**
     * {@link ConfigurationService#isSecurityTopologyPassphraseOk(String)} will return <code>true</code> for the
     * following topology pass-phrase.
     */
    protected final static String VALID_TOPOLOGY_PASSPHRASE = "my-valid-topology-passphrase";

    /**
     * {@link ConfigurationService#isSecurityTopologyPassphraseOk(String)} will return <code>false</code> for the
     * following topology pass-phrase.
     */
    protected final static String INVALID_TOPOLOGY_PASSPHRASE = "my-invalid-topology-passphrase";

    @ClassRule
    public static final PetalsRegistryOverlayNode PETALS_REGISTRY_NODE = new PetalsRegistryOverlayNode();

    /**
     * Create and initialize the mock "Configuration Service"
     */
    protected static ConfigurationService createCfgServiceMock(final String groupName, final String groupPassword,
            final String host, final String port, final String topologyName, final String containerName)
            throws NoContainerConfigurationProvidedException, InvalidContainerConfigurationProvidedException,
            ContainerAlreadyExistException, SubdomainUnknownException {

        final SubDomainConfiguration subDomainConfiguration = AbstractRegistryOverlayImplTest
                .createSubDomainConfiguration(groupName, groupPassword, host, port);

        final DomainConfiguration domainConfiguration = AbstractRegistryOverlayImplTest
                .createDomainConfiguration(topologyName);

        final ContainerConfiguration containerConfiguration = AbstractRegistryOverlayImplTest
                .createContainerConfiguration(containerName);

        return AbstractRegistryOverlayImplTest.createCfgServiceMock(domainConfiguration, subDomainConfiguration,
                containerConfiguration);
    }

    /**
     * Create and initialize the mock "Configuration Service" from a {@link DomainConfiguration}, a
     * {@link SubDomainConfiguration} and a {@link ContainerConfiguration}
     */
    protected static ConfigurationService createCfgServiceMock(final DomainConfiguration domainConfiguration,
            final SubDomainConfiguration subDomainConfiguration, final ContainerConfiguration containerConfiguration)
            throws NoContainerConfigurationProvidedException, InvalidContainerConfigurationProvidedException,
            ContainerAlreadyExistException, SubdomainUnknownException {

        final ConfigurationService cfgServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.makeThreadSafe(cfgServiceMock, true);
        EasyMock.expect(cfgServiceMock.getSubDomainConfiguration()).andReturn(subDomainConfiguration).anyTimes();
        EasyMock.expect(cfgServiceMock.getDomainConfiguration()).andReturn(domainConfiguration).anyTimes();
        EasyMock.expect(cfgServiceMock.getContainerConfiguration()).andReturn(containerConfiguration).anyTimes();
        EasyMock.expect(cfgServiceMock.isSecurityTopologyPassphraseOk(VALID_TOPOLOGY_PASSPHRASE)).andReturn(Boolean.TRUE)
                .anyTimes();
        EasyMock.expect(cfgServiceMock.isSecurityTopologyPassphraseOk(INVALID_TOPOLOGY_PASSPHRASE))
                .andReturn(Boolean.FALSE).anyTimes();
        cfgServiceMock.addContainerConfiguration(containerConfiguration);
        EasyMock.expectLastCall().anyTimes();

        EasyMock.replay(cfgServiceMock);

        return cfgServiceMock;
    }

    protected static DomainConfiguration createDomainConfiguration(final String topologyName) {

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setName(topologyName);

        return domainConfiguration;
    }

    /**
     * Create the configuration
     */
    protected static SubDomainConfiguration createSubDomainConfiguration(final String groupName,
            final String groupPassword, final String host, final String port) {

        return AbstractRegistryOverlayImplTest.createSubDomainConfiguration(groupName, groupPassword, host, port,
                "subdomain-1");
    }

    /**
     * Create the configuration
     */
    protected static SubDomainConfiguration createSubDomainConfiguration(final String groupName,
            final String groupPassword, final String host, final String port, final String subdomainName) {

        // Build the sub-domain configuration
        final SubDomainConfiguration subDomainConfiguration = new SubDomainConfiguration();
        subDomainConfiguration.setName(subdomainName);
        subDomainConfiguration.setDescription("subdomain description");

        final List<Object> extraParameters = new ArrayList<Object>();
        if (groupName != null || groupPassword != null || host != null || port != null) {
            // Build the extra parameter
            final Document extraParameter = DocumentBuilders.newDocument();
            final Element rootElt = extraParameter.createElementNS(ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_ROOT_ELEMENT_NAME);
            final Element topologyNameElt = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_GROUPNAME_ELEMENT_NAME);
            topologyNameElt.setTextContent(groupName);
            rootElt.appendChild(topologyNameElt);
            final Element topologyPwdElt = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_GROUPPWD_ELEMENT_NAME);
            topologyPwdElt.setTextContent(groupPassword);
            rootElt.appendChild(topologyPwdElt);
            final Element overlayMembers = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBERS_ELEMENT_NAME);
            rootElt.appendChild(overlayMembers);
            final Element overlayMember = extraParameter.createElementNS(
                    ConfigurationBuilder.XML.CONFIGURATION_NAMESPACE,
                    ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBER_ELEMENT_NAME);
            overlayMember.setTextContent(host);
            if (port != null) {
                overlayMember.setAttribute(ConfigurationBuilder.XML.CONFIGURATION_OVERLAYMEMBERPORT_ATTR_NAME, port);
            }
            overlayMembers.appendChild(overlayMember);

            extraParameters.add(rootElt);
        }
        subDomainConfiguration.setExtraParameters(extraParameters);

        return subDomainConfiguration;
    }

    protected static ContainerConfiguration createContainerConfiguration(final String containerName) {

        final ContainerConfiguration containerConfiguration = new ContainerConfiguration();
        containerConfiguration.setName(containerName);
        containerConfiguration.setTopologyDynamicLockWaitTime(3);

        return containerConfiguration;
    }

}
