/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.Arrays;

import javax.xml.namespace.QName;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointAlreadyExistsException;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.Location;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;
import org.ow2.petals.microkernel.registry.overlay.api.RegistryOverlayAPI;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;
import com.hazelcast.config.Config;
import com.hazelcast.config.NetworkConfig;
import com.hazelcast.core.Hazelcast;

/**
 * Unit tests of
 * {@link RegistryOverlayImpl#activateEndpoint(QName, String, java.util.List, org.w3c.dom.Document, PetalsServiceEndpoint)}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_ActivateEndpoint_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_1;

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        ENDPOINT_NAME_1 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method
     * {@link RegistryOverlayImpl#activateEndpoint(QName, String, java.util.List, org.w3c.dom.Document, PetalsServiceEndpoint)}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>activate only one valid endpoint.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs, the endpoint is correctly activated</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getEndpoint(QName, String)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForService(QName, LinkType)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForInterface(QName, LinkType)}</li>
     * </ol>
     * </p>
     */
    @Test
    public void testToActivateOneValidEndpoint() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_ActivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToActivateOneValidEndpoint", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final PetalsServiceEndpoint serviceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            EasyMock.expect(serviceEndpointMock.getLocation()).andReturn(new Location()).anyTimes();
            EasyMock.replay(serviceEndpointMock);

            // Add the endpoint
            registryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);

            // Retrieve the endpoint
            final PetalsServiceEndpoint retrievedEndpoint = registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1);
            assertNotNull(retrievedEndpoint);
            assertEquals(retrievedEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint.getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // service
            final PetalsServiceEndpoint[] svcEpForSvc = registryOverlayImpl.getInternalEndpointsForService(SERVICE_NAME_1,
                    LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc);
            assertEquals("No endpoint retrieved for service", svcEpForSvc.length, 1);
            assertEquals(svcEpForSvc[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // interface
            final PetalsServiceEndpoint[] svcEpForItf = registryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf);
            assertEquals("No endpoint retrieved for interface", svcEpForItf.length, 1);
            assertEquals(svcEpForItf[0].getEndpointName(), ENDPOINT_NAME_1);

        } finally {
            registryOverlayImpl.stop();
        }
    }

    /**
     * <p>
     * Check the method
     * {@link RegistryOverlayImpl#activateEndpoint(QName, String, java.util.List, org.w3c.dom.Document, PetalsServiceEndpoint)}:
     * <ol>
     * <li>activate the same endpoint in two different topologies,</li>
     * <li>both topologies use the same registry overlay.</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs, the endpoint is correctly activated in each topology</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getEndpoint(QName, String)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForService(QName, LinkType)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForInterface(QName, LinkType)}</li>
     * </ol>
     * </p>
     */
    @Test
    public void testToActivateTwiceTheSameValidEndpointIn2Topologies() throws Exception {

        // Create a second cluster member in memory on a dedicated port. It will be shutdown by
        // AbstractRegistryOverlayImplTest.stopHazelcastCluster
        final Config hzlConfig = new Config();
        final NetworkConfig networkConfig = new NetworkConfig();
        hzlConfig.setNetworkConfig(networkConfig);
        networkConfig.setPort(Integer.parseInt(RegistryOverlayAPI.DEFAULT_MEMBER_PORT));
        Hazelcast.newHazelcastInstance(hzlConfig);

        final RegistryOverlayImpl registryOverlayImpl_1 = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl_1, "configurationService",
                RegistryOverlayImpl_ActivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToActivateTwiceTheSameValidEndpointIn2Topologies_1", CONTAINER_NAME_1));

        final RegistryOverlayImpl registryOverlayImpl_2 = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl_2, "configurationService",
                RegistryOverlayImpl_ActivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToActivateTwiceTheSameValidEndpointIn2Topologies_2", CONTAINER_NAME_1));

        try {
            registryOverlayImpl_1.start();
            registryOverlayImpl_2.start();

            final PetalsServiceEndpoint serviceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            EasyMock.expect(serviceEndpointMock.getLocation()).andReturn(new Location()).anyTimes();
            EasyMock.replay(serviceEndpointMock);

            // Add the endpoint int topology 1
            registryOverlayImpl_1.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);
            // Add the endpoint int topology 2
            registryOverlayImpl_2.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);

            // Retrieve the endpoint in topology 1
            final PetalsServiceEndpoint retrievedEndpoint_1 = registryOverlayImpl_1.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_1);
            assertNotNull(retrievedEndpoint_1);
            assertEquals(retrievedEndpoint_1.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint_1.getEndpointName(), ENDPOINT_NAME_1);
            // Retrieve the endpoint in topology 2
            final PetalsServiceEndpoint retrievedEndpoint_2 = registryOverlayImpl_2.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_1);
            assertNotNull(retrievedEndpoint_2);
            assertEquals(retrievedEndpoint_2.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint_2.getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // service in topology 1
            final PetalsServiceEndpoint[] svcEpForSvc_1 = registryOverlayImpl_1.getInternalEndpointsForService(
                    SERVICE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc_1);
            assertEquals("No endpoint retrieved for service", svcEpForSvc_1.length, 1);
            assertEquals(svcEpForSvc_1[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // service in topology 2
            final PetalsServiceEndpoint[] svcEpForSvc_2 = registryOverlayImpl_2.getInternalEndpointsForService(
                    SERVICE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc_2);
            assertEquals("No endpoint retrieved for service", svcEpForSvc_2.length, 1);
            assertEquals(svcEpForSvc_2[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // interface in topology 1
            final PetalsServiceEndpoint[] svcEpForItf_1 = registryOverlayImpl_1.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf_1);
            assertEquals("No endpoint retrieved for interface", svcEpForItf_1.length, 1);
            assertEquals(svcEpForItf_1[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the
            // interface in topology 2
            final PetalsServiceEndpoint[] svcEpForItf_2 = registryOverlayImpl_2.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf_2);
            assertEquals("No endpoint retrieved for interface", svcEpForItf_2.length, 1);
            assertEquals(svcEpForItf_2[0].getEndpointName(), ENDPOINT_NAME_1);

        } finally {
            registryOverlayImpl_1.stop();
            registryOverlayImpl_2.stop();
        }
    }

    /**
     * <p>
     * Check the method
     * {@link RegistryOverlayImpl#activateEndpoint(QName, String, java.util.List, org.w3c.dom.Document, PetalsServiceEndpoint)}:
     * <ol>
     * <li>activate the same endpoint twice in the same topology</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>the endpoint is correctly activated on the first one,</li>
     * <li>the error EndpointAlreadyExistsException is thrown on the second activation,</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getEndpoint(QName, String)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForService(QName, LinkType)}</li>
     * <li>the endpoint is retrieve using {@link EndpointRegistry#getInternalEndpointsForInterface(QName, LinkType)}</li>
     * </ol>
     * </p>
     */
    @Test(expected = EndpointAlreadyExistsException.class)
    public void testToActivateTwiceTheSameValidEndpointInTheSameTopology() throws Exception {

        final RegistryOverlayImpl registryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(registryOverlayImpl, "configurationService",
                RegistryOverlayImpl_ActivateEndpoint_Test.createCfgServiceMock(PETALS_REGISTRY_NODE.getGroupName(),
                        PETALS_REGISTRY_NODE.getPassword(), PETALS_REGISTRY_NODE.getHost(),
                        String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass().getSimpleName()
                                + "_testToActivateTwiceTheSameValidEndpointInTheSameTopology", CONTAINER_NAME_1));
        try {
            registryOverlayImpl.start();

            final PetalsServiceEndpoint serviceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            EasyMock.expect(serviceEndpointMock.getLocation()).andReturn(new Location()).anyTimes();
            EasyMock.replay(serviceEndpointMock);

            // 1st Add of the endpoint
            registryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);

            // Retrieve the endpoint
            final PetalsServiceEndpoint retrievedEndpoint = registryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1);
            assertNotNull(retrievedEndpoint);
            assertEquals(retrievedEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint.getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the service
            final PetalsServiceEndpoint[] svcEpForSvc = registryOverlayImpl.getInternalEndpointsForService(SERVICE_NAME_1,
                    LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc);
            assertEquals("No endpoint retrieved for service", svcEpForSvc.length, 1);
            assertEquals(svcEpForSvc[0].getEndpointName(), ENDPOINT_NAME_1);

            // Check that only one endpoint is registered as implementing the interface
            final PetalsServiceEndpoint[] svcEpForItf = registryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf);
            assertEquals("No endpoint retrieved for interface", svcEpForItf.length, 1);
            assertEquals(svcEpForItf[0].getEndpointName(), ENDPOINT_NAME_1);

            // 2nd Add of the endpoint
            try {
                registryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                        Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, serviceEndpointMock, false);
            } catch (final EndpointAlreadyExistsException e) {

                // Retrieve the endpoint
                final PetalsServiceEndpoint retrievedEndpoint2 = registryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                        ENDPOINT_NAME_1);
                assertNotNull(retrievedEndpoint2);
                assertEquals(retrievedEndpoint2.getServiceName(), SERVICE_NAME_1);
                assertEquals(retrievedEndpoint2.getEndpointName(), ENDPOINT_NAME_1);

                // Check that only one endpoint is registered as implementing the service
                final PetalsServiceEndpoint[] svcEpForSvc2 = registryOverlayImpl.getInternalEndpointsForService(
                        SERVICE_NAME_1, LinkType.STANDARD);
                assertNotNull("Null list returned", svcEpForSvc2);
                assertEquals("No endpoint retrieved for service", svcEpForSvc2.length, 1);
                assertEquals(svcEpForSvc2[0].getEndpointName(), ENDPOINT_NAME_1);

                // Check that only one endpoint is registered as implementing the interface
                final PetalsServiceEndpoint[] svcEpForItf2 = registryOverlayImpl.getInternalEndpointsForInterface(
                        INTERFACE_NAME_1, LinkType.STANDARD);
                assertNotNull("Null list returned", svcEpForItf2);
                assertEquals("No endpoint retrieved for interface", svcEpForItf2.length, 1);
                assertEquals(svcEpForItf2[0].getEndpointName(), ENDPOINT_NAME_1);

                throw e;
            }

        } finally {
            registryOverlayImpl.stop();
        }
    }

}
