/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.Arrays;

import javax.xml.namespace.QName;

import org.easymock.EasyMock;
import org.junit.Test;
import org.ow2.petals.jbi.descriptor.original.generated.LinkType;
import org.ow2.petals.microkernel.api.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.microkernel.api.jbi.servicedesc.Location;
import org.ow2.petals.microkernel.api.jbi.servicedesc.PetalsServiceEndpoint;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * Unit tests of {@link RegistryOverlayImpl#removeAllLocalEndpoints()}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayImpl_RemoveAllLocalEndpoints_Test extends AbstractRegistryOverlayImplTest {

    /**
     * A namespace name
     */
    private final static String A_NAMESPACE = "http://petals.ow2.org/unit-tests";

    /**
     * A service name
     */
    private final static QName SERVICE_NAME_1 = new QName(A_NAMESPACE, "service-1");

    /**
     * A interface name
     */
    private final static QName INTERFACE_NAME_1 = new QName(A_NAMESPACE, "interface-1");

    /**
     * A container name
     */
    private final static String CONTAINER_NAME_1 = "container-1";

    /**
     * Another container name
     */
    private final static String CONTAINER_NAME_2 = "container-2";

    /**
     * A endpoint name
     */
    private final static String ENDPOINT_NAME_1;

    /**
     * Another endpoint name
     */
    private final static String ENDPOINT_NAME_2;

    static {
        final QualifiedUUIDGenerator uuidGenerator = new QualifiedUUIDGenerator("petals");
        ENDPOINT_NAME_1 = uuidGenerator.getNewID();
        ENDPOINT_NAME_2 = uuidGenerator.getNewID();
    }

    /**
     * <p>
     * Check the method {@link RegistryOverlayImpl#removeAllLocalEndpoints()}:
     * <ol>
     * <li>in a dedicated topology,</li>
     * <li>an endpoint of local container is registered,</li>
     * <li>an endpoint of a not local container is registered,</li>
     * </ol>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs</li>
     * <li>the endpoint of the local container is no more registered,</li>
     * <li>endpoints of other container is retrieved using {@link EndpointRegistry#getEndpoint(QName, String)}</li>
     * <li>endpoints of other container is retrieved using
     * {@link EndpointRegistry#getInternalEndpointsForService(QName, LinkType)}</li>
     * <li>endpoints of other container is retrieved using
     * {@link EndpointRegistry#getInternalEndpointsForInterface(QName, LinkType)}</li>
     * </ol>
     * </p>
     */
    @Test
    public void testNoLocalEndpointsToRemove() throws Exception {

        final RegistryOverlayImpl localRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(
                localRegistryOverlayImpl,
                "configurationService",
                RegistryOverlayImpl_RemoveAllLocalEndpoints_Test.createCfgServiceMock(
                        PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass()
                                .getSimpleName() + "_testNoLocalEndpointsToRemove", CONTAINER_NAME_1));

        final RegistryOverlayImpl remoteRegistryOverlayImpl = new RegistryOverlayImpl();
        ReflectionHelper.setPrivateField(
                remoteRegistryOverlayImpl,
                "configurationService",
                RegistryOverlayImpl_RemoveAllLocalEndpoints_Test.createCfgServiceMock(
                        PETALS_REGISTRY_NODE.getGroupName(), PETALS_REGISTRY_NODE.getPassword(),
                        PETALS_REGISTRY_NODE.getHost(), String.valueOf(PETALS_REGISTRY_NODE.getPort()), this.getClass()
                                .getSimpleName() + "_testNoLocalEndpointsToRemove", CONTAINER_NAME_2));
        try {
            localRegistryOverlayImpl.start();
            remoteRegistryOverlayImpl.start();

            final PetalsServiceEndpoint localServiceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            final Location localLocation = new Location();
            localLocation.setContainerName(CONTAINER_NAME_1);
            EasyMock.expect(localServiceEndpointMock.getLocation()).andReturn(localLocation).anyTimes();
            EasyMock.replay(localServiceEndpointMock);

            final PetalsServiceEndpoint remoteServiceEndpointMock = EasyMock.createMock(PetalsServiceEndpoint.class);
            final Location remoteLocation = new Location();
            remoteLocation.setContainerName(CONTAINER_NAME_2);
            EasyMock.expect(remoteServiceEndpointMock.getLocation()).andReturn(remoteLocation).anyTimes();
            EasyMock.replay(remoteServiceEndpointMock);

            // Add the local endpoint
            localRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, localServiceEndpointMock, false);

            // Add the remote endpoint
            remoteRegistryOverlayImpl.activateEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_2,
                    Arrays.asList(new QName[] { INTERFACE_NAME_1 }), null, remoteServiceEndpointMock, false);

            // Check that the remote endpoint is visible locally
            final PetalsServiceEndpoint retrievedRemoteEndpoint = localRegistryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_2);
            assertNotNull(retrievedRemoteEndpoint);
            assertEquals(retrievedRemoteEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedRemoteEndpoint.getEndpointName(), ENDPOINT_NAME_2);
            assertEquals(retrievedRemoteEndpoint.getLocation().getContainerName(), CONTAINER_NAME_2);

            // Check the registration of local endpoint
            final PetalsServiceEndpoint retrievedLocalEndpoint = localRegistryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_1);
            assertNotNull(retrievedLocalEndpoint);
            assertEquals(retrievedLocalEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedLocalEndpoint.getEndpointName(), ENDPOINT_NAME_1);
            assertEquals(retrievedLocalEndpoint.getLocation().getContainerName(), CONTAINER_NAME_1);

            // Check that two endpoints are registered as implementing the service
            final PetalsServiceEndpoint[] registeredSvcEpForSvc = localRegistryOverlayImpl.getInternalEndpointsForService(
                    SERVICE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", registeredSvcEpForSvc);
            assertEquals("No endpoint retrieved for service", registeredSvcEpForSvc.length, 2);

            // Check that two endpoints are registered as implementing the interface
            final PetalsServiceEndpoint[] registeredSvcEpForItf = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", registeredSvcEpForItf);
            assertEquals("No endpoint retrieved for interface", registeredSvcEpForItf.length, 2);

            // Remove all local endpoints
            localRegistryOverlayImpl.removeAllLocalEndpoints();

            // Try to retrieve the local endpoint
            assertNull(localRegistryOverlayImpl.getEndpoint(SERVICE_NAME_1, ENDPOINT_NAME_1));

            // Check that the remote endpoint is always visible
            final PetalsServiceEndpoint retrievedEndpoint = localRegistryOverlayImpl.getEndpoint(SERVICE_NAME_1,
                    ENDPOINT_NAME_2);
            assertNotNull(retrievedEndpoint);
            assertEquals(retrievedEndpoint.getServiceName(), SERVICE_NAME_1);
            assertEquals(retrievedEndpoint.getEndpointName(), ENDPOINT_NAME_2);
            assertEquals(retrievedEndpoint.getLocation().getContainerName(), CONTAINER_NAME_2);

            // Check that only one endpoint (remote endpoint) is registered as implementing the service
            final PetalsServiceEndpoint[] svcEpForSvc = localRegistryOverlayImpl.getInternalEndpointsForService(
                    SERVICE_NAME_1,
                    LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForSvc);
            assertEquals("No endpoint retrieved for service", svcEpForSvc.length, 1);
            assertEquals(svcEpForSvc[0].getEndpointName(), ENDPOINT_NAME_2);

            // Check that only one endpoint (remote endpoint) is registered as implementing the interface
            final PetalsServiceEndpoint[] svcEpForItf = localRegistryOverlayImpl.getInternalEndpointsForInterface(
                    INTERFACE_NAME_1, LinkType.STANDARD);
            assertNotNull("Null list returned", svcEpForItf);
            assertEquals("No endpoint retrieved for interface", svcEpForItf.length, 1);
            assertEquals(svcEpForItf[0].getEndpointName(), ENDPOINT_NAME_2);

        } finally {
            remoteRegistryOverlayImpl.stop();
            localRegistryOverlayImpl.stop();
        }
    }

}
